// dllobject.cpp
// example DLL OBJECT, (c) Stewart N. Johnston, Sept. 9, 1993

#include"dllobje.h"
#include<string.h>

/*
Constructor
DLLobject(char *xDLLname,UINT message,WPARAM wParam,LPARAM lParam)
-  xDLLname = the "path\name of dllclass.dll"
-  message,wParam,lParam = Initialization parameters.

This Constructor will:
  - Load the DLL class object.
  - Determine the location of the Interface functions.
  - Construct One instance of the class.
      - Store the DLL instance this pointer in xthis.
  - Set the Status value to its appropriate value.
       -Status = 1  Success_Object_Construction
	       =<0  Error_Code  if it fails
*/

DLLobject::DLLobject(char *xDLLname,UINT message,WPARAM wParam,LPARAM lParam)
    {
    xthis=0;
    _fstrncpy(DLLname,xDLLname,Max_DLL_Name);

    hDLL=LoadLibrary(DLLname);   // Load DLL object

    if(hDLL < 31)                // Exit if no DLL
      {
      Status = Error_Code;
      return;
      }
           // Detemine the location of the Interface functions
    xcon= (CONSTRUCTOR) GetProcAddress(hDLL,MAKEINTRESOURCE(4));
    xmessa= (MESSAFUNCTION) GetProcAddress(hDLL,MAKEINTRESOURCE(3));
    xdest= (DESTRUCTOR) GetProcAddress(hDLL,MAKEINTRESOURCE(2));

    if( !xcon || !xmessa || !xdest )
       {   // Cannot find interface functions
       Status = Error_Code;
       hDLL=0;
       return;
       }
	   // Call constructor interface function
	   // This will construct one instance of the DLL class.
	   // Will return with the this pointer of the created instance.
	   // **** you are now entering the DLL.
     xthis=xcon(message,wParam,lParam);


     if( !xthis)
       {   //  No instance created.
       Status=Error_Code;
       }
     else
       {   //  It has been a success.
       Status=Success_Object_Construction;
       }
     return;
     }

/*
SendMessage(UINT message,WPARAM wParam,LPARAM lParam)

This function will send a message to an already created instance
of a DLL object.
*/

long DLLobject::SendMessage(UINT message,WPARAM wParam,LPARAM lParam)
    {
    
    ULONG X;
        
    if(Status <= Error_Code)
      {
      return 0;
      }
        
    if(!xmessa)
       {
       Status=Error_Code;
       return 0;
       }
	   // Call message interface function.
	   // This will send a message to the instance of the DLL class
	   // which has the this pointer of xthis.
	   // Will return with a 32 bit long response.
	   // **** you are now entering the DLL.
    X=xmessa(xthis,message,wParam,lParam);
     return X;
     }

/*
Destructor
~DLLobject()
Will destroy one instance of class DLL.
Then will free DLL library.
*/

DLLobject::~DLLobject()
    {

     if(Status <= Error_Code)
      {
      return;
      }

    
    if(!xdest || !xthis)
       {    // No interface function
       Status=Error_Code;
       return;
       }
    // Call destuctor interface function.
    // This function will destroy one instance of a DLL class.
    // The xthis parameter must contain the this pointer of the DLL class
    // instance that one seeks to destroy.
    // **** you are now entering the DLL.
    xdest(xthis);

    if(hDLL > 32) FreeLibrary(hDLL); // Free DLL
    }
